; *******************************
; Netflix Silverlight Control v0.7
; *******************************
;
; AutoHotKey script for control of Netflix: Movie Viewer in Internet Explorer

#Persistent
#NoEnv
#SingleInstance

; Initialize window group
ViewerWindowTitle := "Netflix: Netflix Movie Viewer"

GroupAdd, ViewerWindow, %ViewerWindowTitle%



; Init onmessage

OnMessage(0x5555, "HandleMessage")

HandleMessage(wParam, lParam, msg)
{


    if (wParam = 10)
    {

        ; Decide the action to take
        ; (AHK has no switch/case so, we'll have to use else if)
        if (lParam = 100)
        {
            Gosub, CloseViewer ; Exit the netflix movie viewer, restart this script, and wait for a new viewer window to open
        }
        else if (lParam = 101)
        {
            Gosub, JumpAhead
        }
        else if (lParam = 102)
        {
            Gosub, JumpBack ; Jump back in the timeline
        }
        else if (lParam = 103)
        {
            Gosub, NextEpisode ; Move to next Chapter/Episode
        }
        else if (lParam = 104)
        {
            Gosub, PrevEpisode ; Move to previous Chapter/Episode
        }
        else if (lParam = 105)
        {
            Gosub, Play ; Play
        }
        else if (lParam = 106)
        {
            Gosub, Pause ; Pause
        }
        else if (lParam = 107)
        {
            Gosub, TogglePause ;Toggle play/pause
        }
        else if (lParam = 108)
        {
            Gosub, ToggleMute ; Toggle Mute On/Off
        }
        else if (lParam = 109)
        {
            Gosub, VolumeUp ; Move volume Up 
        }
        else if (lParam = 110)
        {
            Gosub, VolumeDown ; Move volume down
        }


    }

    ; Since returning quickly is often important, it is better to use a ToolTip than
    ; something like MsgBox that would prevent the function from finishing:
    ;ToolTip Message %msg% arrived:`nWPARAM: %wParam%`nLPARAM: %lParam%
    ;MsgBox,  Message %msg% arrived:`nWPARAM: %wParam%`nLPARAM: %lParam%
}


; Start of script main loop.
; ***************************
ScriptStart:

; initialize variables
ScriptMode := 0
TimePosition := 0
TimeMovementBuffer := 0
TimeMovementCount := 0
TimeMovementMode := 0

; Wait for a Netflix Movie Viewer window to open
WinWait, ahk_group ViewerWindow

; Set timer to check if window closes
SetTimer, CheckViewerWindow, 100

; Buffer events
#MaxThreadsBuffer On
;
; Wait up to 40 seconds for the "Full screen" button
EnterFullScreen := GetButtonPos("nf_fs.bmp", 74, 34, 400, 0)

; If this a a TV series, there could be chapter buttons.  Find them
PreviousChapter := GetButtonPos("nf_prev_chap.bmp", 41, 24, 1, 0)
NextChapter := GetButtonPos("nf_next_chap.bmp", 41, 24, 1, 0)

; Click the full screen button
result := ClickPos(EnterFullScreen)

; Now that we are in full screen, find the "Exit Full Screen" button
; This will let us know that we are in full screen mode
; Wait about 10 seconds for the full screen button (100 loops of up to 100ms each)
Loop, 100
{
    result =: WiggleMouse()
    ExitFullScreen := GetButtonPos("nf_efs_a.bmp", 74, 32, 1, 45)
        if (ExitFullScreen > 0)
            break
    ExitFullScreen := GetButtonPos("nf_efs_b.bmp", 74, 32, 1, 45)
        if (ExitFullScreen > 0)
            break
    Sleep, 100
}

; now find the pause button
Loop, 100
{
    result =: WiggleMouse()
    Pause := GetButtonPos("nf_pause_a.bmp", 81, 32, 1, 45)
        if (Pause > 0)
           break
    Pause := GetButtonPos("nf_pause_b.bmp", 81, 32, 1, 45)
        if (Pause > 0)
            break
    Pause := GetButtonPos("nf_pause_c.bmp", 81, 32, 1, 45)
        if (Pause > 0)
            break
    Sleep, 100
}

; and now look for the volume button in all its various forms
Loop, 50
{
    result =: WiggleMouse()
    Vol := GetButtonPos("nf_vol_a.bmp", 32, 32, 1, 45)
        if (Vol > 0)
           break
    Vol := GetButtonPos("nf_vol_b.bmp", 32, 32, 1, 45)
        if (Vol > 0)
            break
    Vol := GetButtonPos("nf_vol_c.bmp", 32, 32, 1, 45)
        if (Vol > 0)
            break
    Vol := GetButtonPos("nf_vol_d.bmp", 32, 32, 1, 45)
        if (Vol > 0)
            break
    Sleep, 100
}

; Estimate the width of the timeline bar for later use in jumping forward/back
; We base it off the position of the pause and exit full screen buttons
FarLeftTime := (GetX(Pause) + 56)
TimeLine := (GetY(Pause) - 6)
FarRightTime := (GetX(ExitFullScreen) - 98)

; Let the hot key events know that we have everything set up, and they can now run
ScriptMode := 1

; Move the mouse to the lower right corner
result := HideMouse()

return

; **********************************
; Netflix Viewer Control Subroutines
; **********************************
;
; Call with "Gosub, <name>"
; Example:
; Gosub, VolumeUp
;


; Exit the netflix movie viewer, and wait
; for a new viewer window to open
; ******************************************
CloseViewer:

    ScriptMode := 0
    SetTimer, CheckViewerWindow, Off
    SetTimer, DoTimeScroll, Off
    IfWinExist, ahk_group ViewerWindow
        WinClose, ahk_group ViewerWindow
        WinWaitClose, ahk_group ViewerWindow
    Reload

return

; Jump ahead about 10 pixels in the timeline
; ******************************************
JumpAhead:

    if (ScriptMode < 1)
        return
    TimeMovementBuffer += 10
    SetTimer, DoTimeScroll, 100

return

; Jump back about 10 pixels in the timeline
; ******************************************
JumpBack:

    if (ScriptMode < 1)
        return
    TimeMovementBuffer -= 10
    SetTimer, DoTimeScroll, 100

return

; Move to next Chapter/Episode
; *****************************
NextEpisode:

    if (ScriptMode < 1)
        return
    if (NextChapter > 0)
    {
        CoordMode Mouse
        result := WaitForBar()
        result := ClickPos(ExitFullScreen)
        ; Wait for the chapter button to become visible
        result := GetButtonPos("nf_next_chap.bmp", 41, 24, 100, 0)
        result := ClickPos(NextChapter)
        ; Wait 2 seconds for the new video to load
        sleep 2000
        ; enter back into full screen
        result := GetButtonPos("nf_fs.bmp", 74, 34, 200, 0)
        PreviousChapter := GetButtonPos("nf_prev_chap.bmp", 41, 24, 1, 0)
	NextChapter := GetButtonPos("nf_next_chap.bmp", 41, 24, 1, 0)
        result := ClickPos(EnterFullScreen)
    }

return

; Move to previous Chapter/Episode
; *********************************
PrevEpisode:

    if (ScriptMode < 1)
        return
    if (PreviousChapter > 0)
    {
        CoordMode Mouse
        result := WaitForBar()
        result := ClickPos(ExitFullScreen)
        ; Wait for the chapter button to become visible
        result := GetButtonPos("nf_prev_chap.bmp", 41, 24, 100, 0)
        result := ClickPos(PreviousChapter)
        ; Wait 2 seconds for the new video to load
        sleep 2000
        ; enter back into full screen
        result := GetButtonPos("nf_fs.bmp", 74, 34, 200, 0)
        PreviousChapter := GetButtonPos("nf_prev_chap.bmp", 41, 24, 1, 0)
	NextChapter := GetButtonPos("nf_next_chap.bmp", 41, 24, 1, 0)
        result := ClickPos(EnterFullScreen)
    }

return

; Play
; *****************************
Play:

    if (ScriptMode < 1)
        return
    CoordMode Mouse
    result := WaitForBar()
    Play := GetButtonPos("nf_play.bmp", 81, 32, 1, 0)
    if (Play > 0)
    {
        result := ClickPos(Play)
    }

return

; Pause
; *****************************
Pause:

    if (ScriptMode < 1)
        return
    CoordMode Mouse
    result := WaitForBar()
    Button := GetButtonPos("nf_pause_a.bmp", 81, 32, 1, 45)
        if (Button > 0)
        {
            result := ClickPos(Button)
            return
        }
    Button := GetButtonPos("nf_pause_b.bmp", 81, 32, 1, 45)
        if (Button > 0)
        {
            result := ClickPos(Button)
            return
        }
    Button := GetButtonPos("nf_pause_c.bmp", 81, 32, 1, 45)
        if (Button > 0)
        {
            result := ClickPos(Button)
            return
        }

return

; Toggle play/pause
; *****************************
TogglePause:

    if (ScriptMode < 1)
        return
    CoordMode Mouse
    result := WaitForBar()
    result := ClickPos(Pause)

return

; Toggle Mute On/Off
; *****************************
ToggleMute:

    if (ScriptMode < 1)
        return
    CoordMode Mouse
    result := WaitForBar()
    result := ClickPos(Vol)

return

; Move volume Up
; *****************************
VolumeUp:

    if (ScriptMode < 1)
        return

    VolPos := GetVolumeBarPos(Vol)

    CoordMode Mouse
    if (VolPos > 0)
    {
        x := GetX(Vol)
        NewVolPos := (VolPos - 5)
        MouseMove, %x%, %VolPos%, 5
        MouseClickDrag, Left, x, VolPos, x, NewVolPos, 5
    }

return

; Move volume down
; *****************************
VolumeDown:

    if (ScriptMode < 1)
        return

    VolPos := GetVolumeBarPos(Vol)

    CoordMode Mouse
    if (VolPos > 0)
    {
        x := GetX(Vol)
        NewVolPos := (VolPos + 5)
        MouseMove, %x%, %VolPos%, 3
        MouseClickDrag, Left, x, VolPos, x, NewVolPos, 3
    }

return



; *******************
; Timer Subroutines
; *******************

; Used by check viewer window timer
; Perodically check if the viewer window is open
; If not, re-start from the top
CheckViewerWindow:

    IfWinNotExist, ahk_group ViewerWindow
    {
        ScriptMode := 0
        SetTimer, CheckViewerWindow, Off
        SetTimer, DoTimeScroll, Off
        ;Goto, ScriptStart
        Reload
    }

return


; Used by jump ahead/back hotkeys
DoTimeScroll:

    if ((TimeMovementBuffer = 0) | ((((TimePosition + TimeMovementBuffer) > FarRightTime)  | ((TimePosition + TimeMovementBuffer) < FarLeftTime)) & (TimeMovementMode = 1)))
        {
            TimePosition += TimeMovementBuffer
            TimeMovementBuffer := 0
            if (TimePosition > FarRightTime)
            {
                TimePosition := (FarRightTime)
            }
            if (TimePosition < FarLeftTime)
            {
                TimePosition := (FarLeftTime)
            }
            ; Hiding the mouse, and then repositioning it
            ; keeps the navigation bar active while waiting for
            ; the movment timer to time out
            result := HideMouse()
            CoordMode Mouse
            MouseMove, %TimePosition%, %TimeLine%, 0
            TimeMovementCount += 1
            if (TimeMovementCount > 10)
            {
                CoordMode Mouse
                MouseMove, %TimePosition%, %TimeLine%, 0
                result := WaitForBar()
                CoordMode Mouse
                Click, %TimePosition%, %TimeLine%
                TimeMovementMode := 0
                SetTimer,DoTimeScroll,Off
                result := HideMouse()
            }
        }
        else
        {
            TimeMovementCount  := 0
            if (TimeMovementMode = 0)
            {
                TimeMovementMode := 1
                result := WaitForBar()
                CurPos := GetButtonPos("nf_mark.bmp", 2, 15, 1, 0)
                TimePosition := GetX(CurPos)
            }
            TimePosition += TimeMovementBuffer
            TimeMovementBuffer := 0
            if (TimePosition > FarRightTime)
            {
                TimePosition := (FarRightTime)
            }
            if (TimePosition < FarLeftTime)
            {
                TimePosition := (FarLeftTime)
            }
            CoordMode Mouse
            MouseMove, %TimePosition%, %TimeLine% , 0 
    }
   
return


; ****************************
; Functions
; ****************************


; Look for position on an image
GetButtonPos(g_image,g_width,g_height,g_wait,y_start)
{
    if (y_start > 0)
    {
        y_start := (A_ScreenHeight - 50)
    }

    full_image_path := A_ScriptDir . "\netflix_player_buttons\" . g_image
    reslt = 0
    CoordMode Pixel
    Loop, %g_wait%
    {
        ImageSearch, FoundX, FoundY, 0, %y_start%, A_ScreenWidth, A_ScreenHeight, %full_image_path%
        if ErrorLevel = 0
        {
             ; Pack X,Y coords as two 16 bit integers in one 32 bit word.
             ; We do this because autohotkey does not yet support arrays,
             ; and we can only return one value from this function
             x := (FoundX + (g_width/2))
             y := ((Foundy + (g_height/2)) << 16)
             reslt := (y | x)
             break
        }
        else
        {
            if (g_wait > 1)
                Sleep, 100
        }
    }
    return reslt
}


; Move to position
MoveToPos(position)
{
    x := (position & 65535)
    y := (position >> 16)
    CoordMode Mouse
    MouseMove, %x%, %y%, 0
    return 0
}


; Move mouse to lower right corner of screen
HideMouse()
{
    CoordMode Mouse
    MouseMove, %A_ScreenWidth%, %A_ScreenHeight%, 0
    return 0
}


; Click a position, them move the mouse to the lower right
; corner of the screen 
ClickPos(position){
    x := (position & 65535)
    y := (position >> 16)
    CoordMode Mouse
    MouseMove, %x%, %y%, 0
    CoordMode Mouse
    Click, %x%, %y%
    result := HideMouse()
    return 0
}


; Get the "X" portion of a position
GetX(position)
{
    x := (position & 65535)
    y := (position >> 16)
    return x
}


; Get the "Y" portion of a position
GetY(position)
{
    x := (position & 65535)
    y := (position >> 16)
    return y
}


; Move the mouse cursor around a bit in order to bring up
; the netflix navigation bar that auto hides
WiggleMouse()
{
    wiggle := (A_ScreenWidth - 10)
    CoordMode Mouse
    MouseMove, %A_ScreenWidth%, %A_ScreenHeight%, 2
    ;CoordMode Mouse
    MouseMove, %wiggle%, %A_ScreenHeight%, 2 
    ;CoordMode Mouse
    MouseMove, %A_ScreenWidth%, %A_ScreenHeight%, 2
    ;CoordMode Mouse
    ;MouseMove, %wiggle%, %A_ScreenHeight%, 1 
    return 0
}


; See of the netflix navigation bar is visible
; and bring it up, if not
WaitForBar()
{
    result := LookForVolumeButton()
    if (result = 0)
    {
  
        result := WiggleMouse()

        Loop, 50
        {
            result := LookForVolumeButton()
            if (result = 1)
                break
            Sleep, 50       
        } 
        return result
    }
    return result
}


; See of the volume button is visible
; (if it is, we know the navigation bar is visible)
LookForVolumeButton()
{

   full_image_path_a := A_ScriptDir . "\netflix_player_buttons\nf_vol_a.bmp"
   full_image_path_b := A_ScriptDir . "\netflix_player_buttons\nf_vol_b.bmp"
   full_image_path_c := A_ScriptDir . "\netflix_player_buttons\nf_vol_c.bmp"
   full_image_path_d := A_ScriptDir . "\netflix_player_buttons\nf_vol_d.bmp"

   y_start := (A_ScreenHeight - 50)

       CoordMode Pixel
       ImageSearch, FoundX, FoundY, 0, %y_start%, A_ScreenWidth, A_ScreenHeight, %full_image_path_a%
           if ErrorLevel = 0
               return 1
       ImageSearch, FoundX, FoundY, 0, %y_start%, A_ScreenWidth, A_ScreenHeight, %full_image_path_b%
           if ErrorLevel = 0
               return 1
       ImageSearch, FoundX, FoundY, 0, %y_start%, A_ScreenWidth, A_ScreenHeight, %full_image_path_c%
           if ErrorLevel = 0
               return 1
       ImageSearch, FoundX, FoundY, 0, %y_start%, A_ScreenWidth, A_ScreenHeight, %full_image_path_d%
           if ErrorLevel = 0
               return 1
    return 0

}


; Check if the volume slider bar is visible and return its position
; If its not visible, then make it visible
GetVolumeBarPos(Vol){
    result := LookForVolumeButton()
    if (result = 0)
    {
        result := WaitForBar()
    }

    VolPos := FindVolumeBar(Vol)
    if (VolPos = 0)
    {
        ; Look for volume bar
        result := MoveToPos(Vol)
        Loop, 20
        {
            VolPos := FindVolumeBar(Vol)
            if (VolPos > 0)
                break
            Sleep, 100
        }
    }
    return VolPos
}


; Return the position of the volume slider bar.  
; If its not visible return 0
FindVolumeBar(Vol){

    x := GetX(Vol)
    y := GetY(Vol)
    x1 := (x - 12)
    x2 := (x + 12)
    y1 := (y - 74)
    y2 := (y - 16)
    full_image_path_a := A_ScriptDir . "\netflix_player_buttons\nf_vol_slide_a.bmp"
    full_image_path_b := A_ScriptDir . "\netflix_player_buttons\nf_vol_slide_b.bmp"
    full_image_path_c := A_ScriptDir . "\netflix_player_buttons\nf_vol_slide_c.bmp"
    full_image_path_d := A_ScriptDir . "\netflix_player_buttons\nf_vol_slide_d.bmp"
    full_image_path_e := A_ScriptDir . "\netflix_player_buttons\nf_vol_slide_e.bmp"

    VolPos := 0

        CoordMode Pixel
        ImageSearch, FoundX, FoundY, %x1%, %y1%, %x2%, %y2%, %full_image_path_a%
        if ErrorLevel = 0
        {
            VolPos := (FoundY + 1)
            return VolPos
        }
        ImageSearch, FoundX, FoundY, %x1%, %y1%, %x2%, %y2%, %full_image_path_b%
        if ErrorLevel = 0
        {
            VolPos := (FoundY + 1)
            return VolPos
        }
        ImageSearch, FoundX, FoundY, %x1%, %y1%, %x2%, %y2%, %full_image_path_c%
        if ErrorLevel = 0
        {
            VolPos := (FoundY + 1)
            return VolPos
        }
        ImageSearch, FoundX, FoundY, %x1%, %y1%, %x2%, %y2%, %full_image_path_d%
        if ErrorLevel = 0
        {
            VolPos := (FoundY + 1)
            return VolPos
        }
        ImageSearch, FoundX, FoundY, %x1%, %y1%, %x2%, %y2%, %full_image_path_e%
        if ErrorLevel = 0
        {
            VolPos := (FoundY + 1)
            return VolPos
        }

    return VolPos
}    


